package sample;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.mock.jndi.SimpleNamingContextBuilder;

import com.mchange.v2.c3p0.DriverManagerDataSource;

public class JndiMain {
    
    public static void main(String[] args) throws Exception {
    	// 데이터소스의 오브젝트를 생성
        DriverManagerDataSource ds = new DriverManagerDataSource();
        ds.setDriverClass("org.hsqldb.jdbc.JDBCDriver");
        ds.setJdbcUrl("jdbc:hsqldb:hsql://localhost/sample");
        ds.setUser("sa");
        ds.setPassword("");

        // Spring이 제공하는 JNDI구현(테스트용)을 사용해 네이밍 컨텍스트를 생성
        SimpleNamingContextBuilder builder = new SimpleNamingContextBuilder();
        builder.bind("jdbc/MyDataSource", ds);
        builder.activate();
        
        // Bean 정의 파일을 사용해 JNDI 경유로 데이터소스를 취득 
        ApplicationContext ctx = new ClassPathXmlApplicationContext("spring-jndi.xml");
        JdbcTemplate jdbcTemplate = ctx.getBean(JdbcTemplate.class);
        
        int count = jdbcTemplate.queryForInt("SELECT COUNT(*) FROM PET");
        System.out.println(count);
        
        // 애노테이션을 사용해 JNDI 경유로 데이터소스를 취득 
        JndiAnnotationSample sample = ctx.getBean(JndiAnnotationSample.class);
        sample.printDataSource();
    }
}
